"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const chai = __importStar(require("chai"));
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const nock_1 = __importDefault(require("nock"));
const util_1 = require("../src/util");
const constants_1 = require("./__util__/constants");
chai.use(chai_as_promised_1.default);
const expect = chai.expect;
describe('util', () => {
    describe('queryString()', () => {
        it('should convert object to query string', () => {
            expect((0, util_1.queryString)({ a: '1', b: '2', c: '3' })).to.equal('a=1&b=2&c=3');
        });
    });
    describe('getVQD()', () => {
        it('should get a VQD', async () => {
            const scope = (0, nock_1.default)(constants_1.DDG_HOST).get('/').query({ q: 'node', ia: 'web' }).replyWithFile(200, 'test/__util__/pages/vqd.html');
            await expect((0, util_1.getVQD)('node')).to.eventually.equal(constants_1.SEARCH_QUERY_VQD);
            scope.done();
        });
        it('should error if no VQD was found', async () => {
            const scope = (0, nock_1.default)(constants_1.DDG_HOST).get('/').query({ q: 'node', ia: 'web' }).replyWithFile(200, 'test/__util__/pages/no-vqd.html');
            await expect((0, util_1.getVQD)('node')).to.eventually.be.rejectedWith(Error, 'Failed to get the VQD for query "node".');
            scope.done();
        });
    });
    describe('ensureJSON()', () => {
        it('should parse JSON in buffers', () => {
            expect((0, util_1.ensureJSON)(Buffer.from('{"done":true}'))).to.deep.equal({ done: true });
        });
        it('should parse JSON in strings', () => {
            expect((0, util_1.ensureJSON)('{"done":true}')).to.deep.equal({ done: true });
        });
        it('should passthrough JSON', () => {
            expect((0, util_1.ensureJSON)({ done: true })).to.deep.equal({ done: true });
        });
    });
    describe('parseSpiceBody()', () => {
        it('should parse spice body', () => {
            const body = 'ddg_spice_test(\n{"done":true});\n';
            expect((0, util_1.parseSpiceBody)(body)).to.deep.equal({ done: true });
        });
    });
});
